#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "xsml_lex.h"
#include "xsml_list.h"
#include "xsml_node.h"
#include "xsml_syn.h"

#define MAX_FNAME_LEN		256

#define EI_OPEN_FOUT		-100
#define EI_NODE_TYPE		-101

/* numero dei tipi di token (incluso il tipo sconosciuto) */
#define TOKEN_TYPES_NUM 8

/* stringhe associate ai tipi di token */
static char *token_types[TOKEN_TYPES_NUM] = {
	"unknown token type",
	"ERROR",
	"OPEN_START_TAG",
	"OPEN_END_TAG",
	"CLOSE_TAG",
	"EQUAL",
	"NAME",
	"STRING"
};

/* restituisce la string associata al tipo di token in input */
char *token_type_to_string(int token_type)
{
	/* token sconosciuto */
	if (token_type >= TOKEN_TYPES_NUM-1) return token_types[0];
	
	/* token noto */
	return token_types[token_type+1];
}

int stampa_open_tag(FILE *fp, struct node *nd) {
	int err_code;
	fprintf(fp, "<%s", nd->name);
	attr_it ai = attrlist_it(nd);
	char *attrname = next_attr(ai);
	char *value;
	while (attrname != NULL) {
		err_code = get_attributo(nd, attrname, &value);
		if (err_code) {
			printf("get_attributo() ERROR_CODE = %d\n", err_code);
			return err_code;
		}
		/* non va se attributo contiene carattere ' */
		if (strchr(value, '\'') == NULL)
			fprintf(fp, " %s='%s'", attrname, value);
		else
			fprintf(fp, " %s=\"%s\"", attrname, value);
		attrname = next_attr(ai);
	}
	fprintf(fp, ">");
	return 0;
}

int stampa_content(FILE *fp, struct node *nd) {
	node_it ni = nodelist_it(nd);
	struct node *next = next_node(ni);
	int err_code = OP_OK;
	while (next != NULL && err_code == OP_OK) {
		err_code = stampa_albero(fp, next);
		next = next_node(ni);
	}
	return err_code;
}

int stampa_albero(FILE *fp, struct node *tree) {
	if (tree == NULL)
		return 0;
	if (tree->type == TEXT_NODE)
		fprintf(fp, tree->name);
	else if (tree->type == TAG_NODE) {
		stampa_open_tag(fp, tree);
		stampa_content(fp,tree);
		fprintf(fp, "</%s>", tree->name);
	} else
		return EI_NODE_TYPE;
	return 0;
}

int main(int argc, char **argv)
{
	char fname[MAX_FNAME_LEN];
	int error_code;
	struct node *tree;
	//	lex_an la;
	token tok;
	
	/* se non ci sono parametri, chiede il nome del file da analizzare */
	if (argc == 1) {
		printf("Immetere il nome del file da leggere: ");
		scanf("%s", fname);
	} else if (argc == 2) {
		/* file da analizzare passato come argomento */
		strcpy(fname, argv[1]);
	} else {
		/* numero di argomenti errato */
		printf("Uso: lex_an [<nome_file.xsml>]\n");
		return 1;
	}
	
	/* legge il file e costruisce l'albero del documento*/
	if (error_code = get_xsml_doc(fname, &tree)) {
		/* errrore nella lettura */
		if (error_code == SAERR_BAD_ARG)
			/* inutile cercare di stampare l'ultimo token letto */
			return error_code;
		printf("get_xsml_doc() ERROR_CODE = %d\n", error_code);
		tok = last_token();
		printf("Ultimo token letto: ");
		if (tok.type == CLOSE_TAG || tok.type == EQUAL)
			printf("%d,%d\t%s\n",
				   tok.riga, tok.colonna, token_type_to_string(tok.type));
		else
			printf("%d,%d\t%s\t\"%s\"\n",
				   tok.riga, tok.colonna, token_type_to_string(tok.type), tok.text);
		printf("\n");
		return error_code;
	}
	
	/* stampa il file */
	char foutname[MAX_FNAME_LEN];
	sprintf(foutname, "%s.out", fname);
	FILE *fp = fopen(foutname, "w");
	if (fp == NULL) {
		printf("Impossibile aprire file output: %s\n", foutname);
		return EI_OPEN_FOUT;
	}
	if (error_code = stampa_albero(fp, tree))
		printf("errore stampa albero: %d\n", error_code);
//	fprintf(fp, "\n");
	close(foutname);
	return error_code;
}
